/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2017 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

describe('Notification', function () {

  beforeAll(function (done) {
    // By loading this file it adds the component to the juju components.
    YUI().use('notification', function () {
      done();
    });
  });

  it('renders default', () => {
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.Notification, {
      content: React.createElement(
        'span',
        null,
        'Hello'
      )
    }), true);
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      'div',
      { className: 'p-notification' },
      React.createElement(
        'p',
        { className: 'p-notification__response' },
        React.createElement(
          'span',
          null,
          'Hello'
        )
      )
    );
    expect(output).toEqualJSX(expected);
  });

  it('renders positive', () => {
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.Notification, {
      content: React.createElement('span', null),
      type: 'positive'
    }), true);
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      'div',
      { className: 'p-notification--positive' },
      React.createElement(
        'p',
        { className: 'p-notification__response' },
        React.createElement('span', null)
      )
    );
    expect(output).toEqualJSX(expected);
  });

  it('renders caution', () => {
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.Notification, {
      content: React.createElement('span', null),
      type: 'caution'
    }), true);
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      'div',
      { className: 'p-notification--caution' },
      React.createElement(
        'p',
        { className: 'p-notification__response' },
        React.createElement('span', null)
      )
    );
    expect(output).toEqualJSX(expected);
  });

  it('renders negative', () => {
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.Notification, {
      content: React.createElement('span', null),
      type: 'negative'
    }), true);
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      'div',
      { className: 'p-notification--negative' },
      React.createElement(
        'p',
        { className: 'p-notification__response' },
        React.createElement('span', null)
      )
    );
    expect(output).toEqualJSX(expected);
  });

  it('renders with additional classes', () => {
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.Notification, {
      content: React.createElement('span', null),
      extraClasses: 'test'
    }), true);
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      'div',
      { className: 'p-notification test' },
      React.createElement(
        'p',
        { className: 'p-notification__response' },
        React.createElement('span', null)
      )
    );
    expect(output).toEqualJSX(expected);
  });

  it('renders with dismiss function', () => {
    const dismiss = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.Notification, {
      content: React.createElement('span', null),
      dismiss: dismiss
    }), true);
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      'div',
      { className: 'p-notification' },
      React.createElement(
        'p',
        { className: 'p-notification__response' },
        React.createElement('span', null),
        React.createElement(
          'button',
          { className: 'p-notification__action', onClick: dismiss },
          React.createElement(window.juju.components.SvgIcon, {
            name: 'close_16',
            size: '16' })
        )
      )
    );
    expect(output).toEqualJSX(expected);
  });

  it('renders with a blocking div', () => {
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.Notification, {
      content: React.createElement('span', null),
      isBlocking: true
    }), true);
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      'div',
      { className: 'p-notification__blocker' },
      React.createElement(
        'div',
        { className: 'p-notification' },
        React.createElement(
          'p',
          { className: 'p-notification__response' },
          React.createElement('span', null)
        )
      )
    );
    expect(output).toEqualJSX(expected);
  });

  it('renders with a blocking div and is clickable', () => {
    const dismiss = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.Notification, {
      content: React.createElement('span', null),
      dismiss: dismiss,
      isBlocking: true
    }), true);
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      'div',
      { className: 'p-notification__blocker',
        onClick: dismiss },
      React.createElement(
        'div',
        { className: 'p-notification' },
        React.createElement(
          'p',
          { className: 'p-notification__response' },
          React.createElement('span', null),
          React.createElement(
            'button',
            { className: 'p-notification__action', onClick: dismiss },
            React.createElement(window.juju.components.SvgIcon, {
              name: 'close_16',
              size: '16' })
          )
        )
      )
    );
    expect(output).toEqualJSX(expected);
  });
});